package org.alfresco.bm.event.selector;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.session.SessionService;

import com.google.gson.Gson;

/**
 * Event processor that uses an eventSelector to choose the next event.
 * 
 * @author steveglover
 * @since 1.3
 */
public abstract class AbstractEventSelectorProcessor extends AbstractEventProcessor implements EventDataCreator
{
	private SessionService sessionService;
    private EventSelector eventSelector;
    private boolean persistResponse = false;

	private Gson gson = new Gson();

    public AbstractEventSelectorProcessor(EventSelector eventSelector, SessionService sessionService)
    {
        super();
        this.eventSelector = eventSelector;
        this.sessionService = sessionService;
    }

    /**
     * @param persistResponse          <tt>true</tt> to persist the entire response including the data
     *                                 or <tt>false</tt> just to persist the response message
     */
    public void setPersistResponse(boolean persistResponse) 
    {
        this.persistResponse = persistResponse;
    }

    protected abstract EventProcessorResponse processEventImpl(Object input) throws Exception;

    protected String toJSON(Object obj)
    {
    	String json = gson.toJson(obj);
    	return json;
    }

    private Object getPersistableResponse(EventProcessorResponse response)
    {
    	Object ret = null;

    	if(response != null && response.isPersistAsString())
    	{
    		StringBuilder sb = new StringBuilder();
    		String message = response.getMessage();
    		if(message != null && !message.equals(""))
    		{
    			sb.append(response.getMessage());
    			sb.append(" : ");
    		}
    		sb.append(toJSON(response.getResponseData()));
    		ret = sb.toString();
    	}
    	else
    	{
    		ret = response;
    	}
    	
    	return ret;
    }

    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        Object input = event.getDataObject();
        List<Event> nextEvents = new ArrayList<Event>();

        EventProcessorResponse response = processEventImpl(input);

        // stop the bm timer for this task, the main event processing that we want to time is complete
        super.stopTimer();

        if(response == null)
        {
        	logger.warn("Response is null for event " + event);
        }
        else if(response.isSuccess() && eventSelector != null)
        {
            Object responseData = response.getResponseData();
            Event evt = eventSelector.nextEvent(input, responseData);
            if(evt != null)
            {
                nextEvents.add(evt);
            }
        }

        if(nextEvents.size() < 1)
        {
        	// belt and braces: no more events, close the session if there is one
        	String sessionId = event.getSessionId();
            if(sessionId != null)
            {
                sessionService.endSession(sessionId);
            }
        }
        
        Object data = (persistResponse ? getPersistableResponse(response) : response.getMessage());
        EventResult result = new EventResult(
                data,
                nextEvents,
                response.isSuccess());
        return result;
    }
}
