/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.file;

import org.alfresco.bm.user.UserData;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

import com.mongodb.BasicDBObjectBuilder;
import com.mongodb.DBObject;
import com.mongodb.WriteConcern;

/**
 * Concrete service implementation of {@link FileDataService} based on MongoDB.
 *
 * @author Derek Hulley
 * @since 1.4
 */
public class FileDataServiceImpl implements FileDataService, InitializingBean
{
    private MongoTemplate mongo;
    private String collection;
    
    public FileDataServiceImpl(MongoTemplate mongo, String collection)
    {
        this.mongo = mongo;
        this.collection = collection;
    }
    
    @Override
    public void afterPropertiesSet() throws Exception
    {
        checkIndexes();
    }

    /**
     * Ensure that the MongoDB collection has the required indexes associated with
     * this user bean.
     */
    private void checkIndexes()
    {
        mongo.getDb().getCollection(collection).setWriteConcern(WriteConcern.SAFE);
        
        DBObject idxRandomizer = BasicDBObjectBuilder
                .start(FileData.FIELD_FILESET, 1)
                .add(FileData.FIELD_RANDOMIZER, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(idxRandomizer, "idx_randomizer", false);

        DBObject idxExtension = BasicDBObjectBuilder
                .start(FileData.FIELD_FILESET, 1)
                .add(FileData.FIELD_EXTENSION, 1)
                .add(FileData.FIELD_RANDOMIZER, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(idxExtension, "idx_extension", false);

        DBObject uidxRemoteName = BasicDBObjectBuilder
                .start(FileData.FIELD_FILESET, 1)
                .add(FileData.FIELD_REMOTE_NAME, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(uidxRemoteName, "uidx_remoteName", true);

        DBObject uidxLocalName = BasicDBObjectBuilder
                .start(FileData.FIELD_FILESET, 1)
                .add(FileData.FIELD_LOCAL_NAME, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(uidxLocalName, "uidx_localName", true);
    }
    
    @Override
    public void createNewFileData(FileData fileData)
    {
        mongo.insert(fileData, collection);
    }

    @Override
    public long fileCount(String fileset)
    {
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset);
        Query query = new Query(criteria);
        return mongo.count(query, collection);
    }

    @Override
    public FileData findFile(String fileset, String remoteName)
    {
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset)
                .and(FileData.FIELD_REMOTE_NAME).is(remoteName);
        Query query = new Query(criteria);
        
        FileData result = mongo.findOne(query, FileData.class, collection);
        return result;
    }

    @Override
    public void removeFile(String fileset, String remoteName)
    {
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset)
                .and(FileData.FIELD_REMOTE_NAME).is(remoteName);
        Query query = new Query(criteria);
        
        mongo.remove(query, collection);
    }

    /**
     * @return              the maximum value of the randomizer for the given constraints
     */
    private int getMaxRandomizer(String fileset)
    {
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset);
        Query query = new Query(criteria)
                .with(new Sort(Direction.DESC, UserData.FIELD_RANDOMIZER));
        query.limit(1);
        
        FileData firstResult = (FileData) mongo.findOne(query, FileData.class, collection);
        return firstResult == null ? 0 : firstResult.getRandomizer();
    }

    @Override
    public FileData getRandomFile(String fileset)
    {
        int maxRandomizer = getMaxRandomizer(fileset);
        int random = (int) (Math.random() * (double) maxRandomizer);
        
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset)
                .and(FileData.FIELD_RANDOMIZER).gte(new Integer(random));
        Query query = new Query(criteria)
                .with(new Sort(FileData.FIELD_RANDOMIZER))
                .with(new PageRequest(0, 1));

        FileData firstResult = (FileData) mongo.findOne(query, FileData.class, collection);
        return firstResult;
    }

    /**
     * @return              the maximum value of the randomizer for the given constraints
     */
    private int getMaxRandomizer(String fileset, String extension)
    {
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset)
                .and(FileData.FIELD_EXTENSION).is(extension);
        Query query = new Query(criteria)
                .with(new Sort(Direction.DESC, UserData.FIELD_RANDOMIZER));
        query.limit(1);
        
        FileData firstResult = (FileData) mongo.findOne(query, FileData.class, collection);
        return firstResult == null ? 0 : firstResult.getRandomizer();
    }

    @Override
    public FileData getRandomFile(String fileset, String extension)
    {
        int maxRandomizer = getMaxRandomizer(fileset, extension);
        int random = (int) (Math.random() * (double) maxRandomizer);
        
        Criteria criteria = Criteria
                .where(FileData.FIELD_FILESET).is(fileset)
                .and(FileData.FIELD_EXTENSION).is(extension)
                .and(FileData.FIELD_RANDOMIZER).gte(new Integer(random));
        Query query = new Query(criteria)
                .with(new Sort(FileData.FIELD_RANDOMIZER))
                .with(new PageRequest(0, 1));

        FileData firstResult = (FileData) mongo.findOne(query, FileData.class, collection);
        return firstResult;
    }
}
