/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.log;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

import com.mongodb.BasicDBObjectBuilder;
import com.mongodb.DBObject;

/**
 * An {@link LogService} <b>MongoDB</b> implementation
 * 
 * @author Derek Hulley
 * @since 1.4
 */
public class MongoLogService implements LogService
{
    private static Log logger = LogFactory.getLog(MongoLogService.class);

    private final MongoTemplate mongo;
    private final String collection;
    private final String clusterUUID;

    /**
     * Construct a provider that works using the given Mongo instance and
     * collection name.
     * 
     * @param mongo the MongoDB template to use
     * @param collection the name of the MongoDB collection
     */
    public MongoLogService(MongoTemplate mongo, String collection, String clusterUUID)
    {
        this.mongo = mongo;
        this.collection = collection;
        this.clusterUUID = clusterUUID;
        // Initialize indexes
        DBObject idxSeverity = BasicDBObjectBuilder
                .start(LogMessage.FIELD_CLUSTER_UUID, 1)
                .add(LogMessage.FIELD_TESTRUN_FQN, 1)
                .add(LogMessage.FIELD_SEVERITY, 1)
                .add(LogMessage.FIELD_TIME, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(idxSeverity, LogMessage.INDEX_SEVERITY, false);
        
        DBObject idxServer = BasicDBObjectBuilder
                .start(LogMessage.FIELD_CLUSTER_UUID, 1)
                .add(LogMessage.FIELD_TESTRUN_FQN, 1)
                .add(LogMessage.FIELD_SERVER_ID, 1)
                .add(LogMessage.FIELD_TIME, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(idxServer, LogMessage.INDEX_SERVER, false);
        
        DBObject idxTime = BasicDBObjectBuilder
                .start(LogMessage.FIELD_CLUSTER_UUID, 1)
                .add(LogMessage.FIELD_TESTRUN_FQN, 1)
                .add(LogMessage.FIELD_TIME, 1)
                .get();
        mongo.getDb().getCollection(collection).ensureIndex(idxTime, LogMessage.INDEX_TIME, false);
    }

    @Override
    public void log(String serverId, String testRunFQN, int severity, String msg)
    {
        LogMessage logMessage = new LogMessage(
                clusterUUID, serverId, testRunFQN, severity,
                System.currentTimeMillis(),
                msg);
        log(logMessage);
    }

    /**
     * Records and logs the given message
     */
    private void log(LogMessage logMessage)
    {
        switch (logMessage.getSeverity())
        {
            case 0:
                if (logger.isTraceEnabled())
                {
                    logger.trace(logMessage);
                }
                break;
            case 1:
                if (logger.isDebugEnabled())
                {
                    logger.debug(logMessage);
                }
                break;
            case 2:
                if (logger.isInfoEnabled())
                {
                    logger.info(logMessage);
                }
                break;
            case 3:
                logger.warn(logMessage);
                break;
            case 4:
                logger.error(logMessage);
                break;
            default:
                logger.fatal(logMessage);
        }
        mongo.insert(logMessage, collection);
    }

    @Override
    public List<LogMessage> getLogs(
            String testRunFQN,
            String serverId,
            int severity,
            Long minTime, Long maxTime,
            int skip, int limit)
    {
        Criteria criteria = Criteria
                .where(LogMessage.FIELD_CLUSTER_UUID).is(clusterUUID)
                .and(LogMessage.FIELD_TESTRUN_FQN).is(testRunFQN)
                .and(LogMessage.FIELD_SEVERITY).gte(severity);
        if (serverId != null)
        {
            criteria = criteria.and(LogMessage.FIELD_SERVER_ID).is(serverId);
        }
        if (minTime != null)
        {
            criteria = criteria.and(LogMessage.FIELD_TIME).gte(minTime);
        }
        if (maxTime != null)
        {
            criteria = criteria.and(LogMessage.FIELD_TIME).lt(maxTime);
        }
        
        Query query = new Query(criteria).skip(skip).limit(limit);
        query.with(new Sort(Direction.ASC, LogMessage.FIELD_TIME));
        
        List<LogMessage> results = mongo.find(query, LogMessage.class, collection);
        return results;
    }
}
